import argparse
import os
from scipy.misc import imread, imresize
import face_tool
from storage import Storage
import logger
from shutil import copyfile

ap = argparse.ArgumentParser()
ap.add_argument("-i", "--input", required=True,
                help="input directory that contains image files.")
ap.add_argument("-o", "--output", required=True,
                help="output directory.")
ap.add_argument("-s", "--scale", default=50, type=int,
                help="reduce image size as percentage.")
ap.add_argument("-t", "--threshold", default=0.5, type=float,
                help="threshold of face recognizer.")
ap.add_argument("-d", "--database", default=os.path.join(os.environ['HOME'], ".face_cluster"), type=str,
                help="place to save encoded face information.")
args = vars(ap.parse_args())

faces = []

logger.info("Reading storage ...")
storage = Storage(args["database"])
if storage.has_faces():
    logger.info("Scanning known faces ...")
    known_faces_list = storage.scan_faces()
    known_faces = []
    for known_face_encoding, known_face_id in known_faces_list:
        face = face_tool.Face(known_face_encoding, id=known_face_id, cache=True)
        known_faces.append(face)
    logger.info("{} faces has been loaded from database.".format(len(known_faces)))
    faces += known_faces


def scan_images(path):
    logger.info("Scanning images ...")
    output = []
    for root, dirs, files in os.walk(path):
        for f in files:
            if f.endswith(".jpg"):
                output.append(os.path.join(root, f))
    return output


def process_files(files):
    for f in files:
        logger.info("Reading {}".format(f), newline=False)
        image = imread(f)
        image = imresize(image, args["scale"])
        detected_faces = face_tool.detect_faces(image)
        landmarks = face_tool.face_landmarks(image, detected_faces)
        encodings = face_tool.face_encodings(image, landmarks)

        prepared_known_faces = [face.encoding for face in faces]
        labels = []
        for face_encoding in encodings:
            index, _ = face_tool.compare_faces(prepared_known_faces, face_encoding)
            face = face_tool.Face(face_encoding, path=f)
            if index >= 0:
                face.id = faces[index].id
            labels.append(str(face.id))
            faces.append(face)
        logger.info(". Done", prefix=False)
        logger.info("Details , {} faces , labels {}".format(len(detected_faces), labels))


def main():
    process_files(scan_images(args["input"]))
    if not os.path.exists(args["output"]):
        os.mkdir(args["output"])

    face_counter = {}
    logger.info("Saving faces, {} detected".format(len(faces)))
    for face in faces:
        if not face.cache:
            storage.save_face(face)

            target_path = os.path.join(args["output"], str(face.id))
            if not os.path.exists(target_path):
                os.mkdir(target_path)

            number = 0
            if face.id in face_counter:
                number = face_counter[face.id]
            copyfile(face.path, os.path.join(target_path, "{}.jpg".format(number)))
            face_counter[face.id] = number + 1


if __name__ == "__main__":
    main()



import os
import numpy as np
import time
import glob


class Storage:
    def __init__(self, path):
        self.path = path
        if not os.path.exists(path):
            os.mkdir(path)

    def has_faces(self):
        return len(glob.glob(os.path.join(self.path, "*.dump")))

    def scan_faces(self):
        output = []
        for dump_file in glob.glob(os.path.join(self.path, "*.dump")):
            face_id = dump_file[dump_file.rindex("/") + 1:-5][:36]
            face_encoding = np.loadtxt(dump_file)
            output.append([face_encoding, face_id])

        return output

    def save_face(self, face):
        prefix = "{}".format(face.id)
        number = len(glob.glob(os.path.join(self.path, prefix + "*.dump")))
        np.savetxt(os.path.join(self.path, "{}-{}-{}.dump".format(prefix, int(time.time() / 10000) * 10, number)),
                   face.encoding)



import sys

# TODO: detect os and replace \n with \r\n if was windows.
new_line_char = "\n"


def change_stdout(stream=None):
    sys.stdout = stream


def info(string_input, newline=True, prefix=True):
    sys.stdout.write("{}{}{}".format("[INFO] " if prefix else "", string_input, new_line_char if newline else ""))


def warn(string_input, newline=True):
    sys.stdout.write("[WARN] {}{}".format(string_input, new_line_char if newline else ""))


def error(string_input, newline=True):
    sys.stderr.write("[ERROR] {}{}".format(string_input, new_line_char if newline else ""))



from setuptools import setup

setup(
    name="orcas",
    version="0.0.1",
    description="Classify the images in your photo album into suitable destinations.",
    url="http://github.com/ahmdrz/orcas",
    author="Ahmadreza Zibaei",
    author_email="ahmadrezazibaei@hotmail.com",
    license='MIT',
    packages=["orcas"],
    entry_points={
        'console_scripts': [
            'orcas=orcas.cli:main'
        ]
    },
    install_requires=[
        'dlib',
        'scipy',
        'numpy'
    ],
    package_dir={'orcas': 'orcas'},
    package_data={
        'orcas': ['data/*.dat']
    },
    include_package_data=True,
    zip_safe=False
)



import dlib
import uuid
import numpy as np
import os

_base_dir = os.path.dirname(__file__)

_detector = dlib.get_frontal_face_detector()
_predictor = dlib.shape_predictor(os.path.join(os.path.join(_base_dir, "data"), "shape_predictor_5_face_landmarks.dat"))
_encoder = dlib.face_recognition_model_v1(os.path.join(os.path.join(_base_dir, "data"), "dlib_face_recognition_resnet_model_v1.dat"))


def detect_faces(img):
    return _detector(img, 1)


def face_landmarks(img, locations):
    return [_predictor(img, location) for location in locations]


def face_encodings(img, locations):
    return [np.array(_encoder.compute_face_descriptor(img, location, 1)) for
            location in locations]


def face_distance(encoded_faces, face_to_compare):
    if len(encoded_faces) == 0:
        return np.empty(0)

    return np.linalg.norm(encoded_faces - face_to_compare, axis=1)


def compare_faces(known_face_encodings, face_encoding_to_check, threshold=0.5):
    if not known_face_encodings:
        return -1, 0.0

    matches = list(face_distance(known_face_encodings, face_encoding_to_check))
    index = np.argmin(matches)
    if matches[index] < threshold:
        return index, matches[index]
    return -1, 0.0


def cluster_faces(descriptors, threshold=0.5):
    return dlib.chinese_whispers_clustering(descriptors, threshold)


class Face:
    def __init__(self, encoding, id=None, cache=False, path=None):
        if not id:
            self.id = uuid.uuid4()
        else:
            self.id = id

        if not path and not cache:
            raise RuntimeError("face is not cache and has not path")
        self.path = path
        self.cache = cache
        self.encoding = encoding
